package com.aoc.psisclient.base;

import javax.xml.namespace.QName;

import oasis.names.tc.dss._1_0.core.wsdl.DigitalSignatureService;
import oasis.names.tc.dss._1_0.core.wsdl.SOAPport;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.aoc.psisclient.config.PsisClientFactoryConfigurable;

public class PsisClientFactory {
	private static Logger log = LogManager.getLogger(PsisClientFactory.class);
	private static QName psisDssPortName = new QName("urn:service", "dssPortSoap");
	private static QName psisDssPdfPortName = new QName("urn:service", "dssPDFPortSoap");

	private DigitalSignatureService service;
	private PsisClientFactoryConfigurable configurator;

	public PsisClientFactoryConfigurable getConfigurator() {
		return this.configurator;
	}

	public void setConfigurator(PsisClientFactoryConfigurable configurator) {
		this.configurator = configurator;
		init();
	}

	public SOAPport getDssPort(){
		log.info("Getting DSS soap port");
		SOAPport port = this.service.getPort(PsisClientFactory.psisDssPortName, SOAPport.class);
		log.info("DSS soap port created succesfully!");
		return port;
	}

	public SOAPport getDssPdfPort(){
		log.info("Getting DSS_PDF soap port");
		SOAPport port = this.service.getPort(PsisClientFactory.psisDssPdfPortName, SOAPport.class);
		log.info("DSS_PDF soap port created succesfully!");
		return port;
	}

	private void init(){
		log.info("Starting creating service");	
		log.info("DSS Endpoint: " + this.getConfigurator().getDSSEndpoint().toString());
		log.info("DSSPDF Endpoint: " + this.getConfigurator().getDSSPDFEndpoint().toString());
		this.service = new DigitalSignatureService();
		this.service.addPort(PsisClientFactory.psisDssPortName, "http://schemas.xmlsoap.org/soap/", this.getConfigurator().getDSSEndpoint().toString());
		this.service.addPort(PsisClientFactory.psisDssPdfPortName,"http://schemas.xmlsoap.org/soap/", this.getConfigurator().getDSSPDFEndpoint().toString());
		log.info("Service created succesfully!");
	}
}
