package com.aoc.psisclient.samples;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.List;

import org.apache.commons.io.FileUtils;

import com.aoc.psisclient.base.AbstractSample;

import x0CoreSchema.oasisNamesTcDss1.Base64DataDocument.Base64Data;
import x0CoreSchema.oasisNamesTcDss1.DocumentWithSignatureDocument.DocumentWithSignature;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;
import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.OptionalOutputsDocument.OptionalOutputs;
import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument;
import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument.VerifyRequest;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument;

public class PdfValidationSample extends AbstractSample {

	public PdfValidationSample() {
		super();
	}

	public static void main(String[] args) throws IOException {
		File pdf = new File("src\\pdf\\pdf_signed.pdf");
		PdfValidationSample validation = new PdfValidationSample();
		validation.checkPdfAndShowRessult(pdf);
	}

	public void checkPdfAndShowRessult(File pdf) throws IOException {
		VerifyResponseDocument response = verifyPdf(pdf);
		if (isRessultWellFormed(response)) {
			showMinorMajorRessult(response);
		}
		OptionalOutputs optionalOutputs = response.getVerifyResponse().getOptionalOutputs();
		if (optionalOutputs != null) {
			List<Calendar> expirationDates = optionalOutputs.getPDFExpirationDateList();
			if (expirationDates != null && expirationDates.size() == 1) {
				Calendar expirationDate = optionalOutputs.getPDFExpirationDateBeforeUpdateList().get(0);
				this.log.info("PDFExpirationDateBeforeUpdate: " + expirationDate);
			}
			if (expirationDates != null && expirationDates.size() == 1) {
				Calendar expirationDate = optionalOutputs.getPDFExpirationDateList().get(0);
				this.log.info("PDFExpirationDate: " + expirationDate);
			}
			List<BigInteger> totalRevisions = optionalOutputs.getPDFTotalRevisionsNumberList();
			if (totalRevisions != null && totalRevisions.size() == 1) {
				this.log.info("PDFTotalRevisionsNumber: " + totalRevisions.get(0));
			}
			List<DocumentWithSignature> updatedSignatureList = optionalOutputs.getDocumentWithSignatureList();
			if(updatedSignatureList != null && updatedSignatureList.size() == 1) {
				DocumentWithSignature updatedSignature = updatedSignatureList.get(0);
				byte[] updatedPdfBA = updatedSignature.getDocument().getBase64Data().getByteArrayValue();
				File updatedPdfFile = new File("src\\pdf\\pdf_updated.pdf");
				FileOutputStream fos = new FileOutputStream(updatedPdfFile);
				fos.write(updatedPdfBA);
				fos.close();
				this.log.info("Updated PDF saved to: " + updatedPdfFile.getAbsolutePath());
			}
		}
	}

	public VerifyResponseDocument verifyPdf(File pdf) throws IOException {
		this.log.info("Start validation " + PDF_PROFILE);
		VerifyResponseDocument response = null;
		this.log.debug("Start reading File " + pdf.getAbsoluteFile());
		byte[] bytes = FileUtils.readFileToByteArray(pdf);

		this.log.debug("Start message creation ");
		VerifyRequest request = VerifyRequest.Factory.newInstance();
		request.setProfile(PDF_PROFILE);
		InputDocuments inputDocument = InputDocuments.Factory.newInstance();
		Base64Data base64Data = Base64Data.Factory.newInstance();
		base64Data.setByteArrayValue(bytes);
		base64Data.setMimeType(MIME_TYPE_APPLICATION_PDF);
		inputDocument.addNewDocument().setBase64Data(base64Data);
		request.setInputDocuments(inputDocument);
		OptionalInputs optionalInputs = OptionalInputs.Factory.newInstance();
		// If we want to update to LTV (Long term validation)
		optionalInputs.addNewReturnUpdatedSignature().setType(PADES_LTV_TYPE);
		// If we want to know total number of revisions
		optionalInputs.addNewReturnPDFTotalRevisionsNumber();
		// If we want to know pdf Expiration Date
		optionalInputs.addNewReturnPDFExpirationDate();
		// If we want to know the pdf ExpirationdateBeforeUpdate
		optionalInputs.addNewReturnPDFExpirationDateBeforeUpdate();

		this.log.debug("Adding optional Inputs\n" + optionalInputs.toString());
		request.setOptionalInputs(optionalInputs);
		VerifyRequestDocument verifyRequest = VerifyRequestDocument.Factory.newInstance();
		verifyRequest.setVerifyRequest(request);

		this.log.debug("Send request pdf verification");
		response = this.factory.getDssPdfPort().verify(verifyRequest);
		return response;
	}
}
