package com.aoc.psisclient.samples;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.MessageDigest;

import org.apache.commons.io.FileUtils;
import org.apache.xmlbeans.XmlException;
import org.w3.x2000.x09.xmldsig.DigestMethodType;

import x0CoreSchema.oasisNamesTcDss1.DocumentHashDocument.DocumentHash;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;
import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.SignatureObjectType;
import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument;
import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument.VerifyRequest;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument;

import com.aoc.psisclient.base.AbstractSample;

public class XadesSignatureValidationSample extends AbstractSample {

	public static void main(String[] args) {
		try {
			XadesSignatureValidationSample validation = new XadesSignatureValidationSample();
			FileInputStream fis = new FileInputStream("src\\signatures\\xades.xml");
			MessageDigest digester = MessageDigest.getInstance("SHA-1");
			byte[] doc = FileUtils.readFileToByteArray(new File("src\\signatures\\doc.txt"));
			byte[] digest = digester.digest(doc);
			validation.checkSignatureAndShowRessult(fis, digest);
		} catch (Exception e) {
			System.out.print(e);
		}
	}

	public void checkSignatureAndShowRessult(FileInputStream fis, byte[] digest) throws XmlException, IOException {
		VerifyResponseDocument response = verifyXAdESSignature(fis, digest);
		if (isRessultWellFormed(response)) {
			showMinorMajorRessult(response);
			showOptionalOutputs(response);
		}
	}

	public VerifyResponseDocument verifyXAdESSignature(FileInputStream fis, byte[] digest) throws XmlException, IOException {
		this.log.debug("Start validation " + XML_PROFILE);
		VerifyRequest request = VerifyRequest.Factory.newInstance();
		request.setProfile(XML_PROFILE);

		this.log.debug("Parsing XADES signature");
		SignatureObjectType signatureObjectType = SignatureObjectType.Factory.parse(fis);
		request.setSignatureObject(signatureObjectType);

		this.log.debug("Adding diggest");
		InputDocuments inputDocument = request.addNewInputDocuments();
		DocumentHash documentHash = inputDocument.addNewDocumentHash();
		documentHash.setDigestValue(digest);
		DigestMethodType digestAlgorithm = DigestMethodType.Factory.newInstance();
		digestAlgorithm.setAlgorithm(DIGEST_ALGORITHM_SHA1);
		documentHash.setDigestMethod(digestAlgorithm);

		this.log.debug("Adding optionalInput UPDATE signature to XAdES A");
		OptionalInputs optionalInput = request.addNewOptionalInputs();
		optionalInput.addNewReturnUpdatedSignature().setType(UPDATE_TYPE_XADES_A);

		this.log.debug("Send request XML verification");
		VerifyRequestDocument verifyRequest = VerifyRequestDocument.Factory.newInstance();
		verifyRequest.setVerifyRequest(request);
		VerifyResponseDocument response = this.factory.getDssPort().verify(verifyRequest);
		return response;
	}
}
